@@ -11,11 +11,12 @@ use warnings;
 
 no warnings qw( uninitialized once );
 
-our $VERSION = '1.843';
+our $VERSION = '1.865';
 
 use base 'MCE::Channel';
 use bytes;
 
+my $LF = "\012"; Internals::SvREADONLY($LF, 1);
 my $is_MSWin32 = ( $^O eq 'MSWin32' ) ? 1 : 0;
 my $freeze     = MCE::Channel::_get_freeze();
 my $thaw       = MCE::Channel::_get_thaw();
@@ -39,6 +40,7 @@ sub end {
    my ( $self ) = @_;
    return if $self->{ended};
 
+   local $\ = undef if (defined $\);
    MCE::Util::_sock_ready_w( $self->{p_sock} ) if $is_MSWin32;
    print { $self->{p_sock} } pack('i', -1);
 
@@ -49,6 +51,7 @@ sub enqueue {
    my $self = shift;
    return MCE::Channel::_ended('enqueue') if $self->{ended};
 
+   local $\ = undef if (defined $\);
    MCE::Util::_sock_ready_w( $self->{p_sock} ) if $is_MSWin32;
 
    while ( @_ ) {
@@ -68,9 +71,10 @@ sub dequeue {
    my ( $self, $count ) = @_;
    $count = 1 if ( !$count || $count < 1 );
 
+   local $/ = $LF if ( $/ ne $LF );
+
    if ( $count == 1 ) {
       my ( $plen, $data );
-
       MCE::Util::_sock_ready( $self->{c_sock} ) if $is_MSWin32;
 
       $is_MSWin32
@@ -93,7 +97,6 @@ sub dequeue {
    }
    else {
       my ( $plen, @ret );
-
       MCE::Util::_sock_ready( $self->{c_sock} ) if $is_MSWin32;
 
       while ( $count-- ) {
@@ -125,10 +128,10 @@ sub dequeue_nb {
    $count = 1 if ( !$count || $count < 1 );
 
    my ( $plen, @ret );
+   local $/ = $LF if ( $/ ne $LF );
 
    while ( $count-- ) {
       my $data;
-
       MCE::Util::_nonblocking( $self->{c_sock}, 1 );
 
       $is_MSWin32
@@ -170,6 +173,7 @@ sub send {
       $data = $_[0], $data .= '0';
    }
 
+   local $\ = undef if (defined $\);
    MCE::Util::_sock_ready_w( $self->{p_sock} ) if $is_MSWin32;
    print { $self->{p_sock} } pack('i', length $data) . $data;
 
@@ -180,6 +184,7 @@ sub recv {
    my ( $self ) = @_;
    my ( $plen, $data );
 
+   local $/ = $LF if ( $/ ne $LF );
    MCE::Util::_sock_ready( $self->{c_sock} ) if $is_MSWin32;
 
    $is_MSWin32
@@ -205,6 +210,7 @@ sub recv_nb {
    my ( $self ) = @_;
    my ( $plen, $data );
 
+   local $/ = $LF if ( $/ ne $LF );
    MCE::Util::_nonblocking( $self->{c_sock}, 1 );
 
    $is_MSWin32
@@ -244,8 +250,17 @@ sub send2 {
       $data = $_[0], $data .= '0';
    }
 
-   MCE::Util::_sock_ready_w( $self->{c_sock} ) if $is_MSWin32;
-   print { $self->{c_sock} } pack('i', length $data) . $data;
+   local $\ = undef if (defined $\);
+   local $MCE::Signal::SIG;
+
+   {
+      local $MCE::Signal::IPC = 1;
+
+      MCE::Util::_sock_ready_w( $self->{c_sock} ) if $is_MSWin32;
+      print { $self->{c_sock} } pack('i', length $data) . $data;
+   }
+
+   CORE::kill($MCE::Signal::SIG, $$) if $MCE::Signal::SIG;
 
    return 1;
 }
@@ -254,6 +269,7 @@ sub recv2 {
    my ( $self ) = @_;
    my ( $plen, $data );
 
+   local $/ = $LF if ( $/ ne $LF );
    MCE::Util::_sock_ready( $self->{p_sock} ) if $is_MSWin32;
 
    $is_MSWin32
@@ -275,6 +291,7 @@ sub recv2_nb {
    my ( $self ) = @_;
    my ( $plen, $data );
 
+   local $/ = $LF if ( $/ ne $LF );
    MCE::Util::_nonblocking( $self->{p_sock}, 1 );
 
    $is_MSWin32
@@ -311,18 +328,62 @@ MCE::Channel::Simple - Channel tuned for one producer and one consumer
 
 =head1 VERSION
 
-This document describes MCE::Channel::Simple version 1.843
+This document describes MCE::Channel::Simple version 1.865
 
 =head1 DESCRIPTION
 
 A channel class providing queue-like and two-way communication
 for one process or thread on either end; no locking needed.
 
+The API is described in L<MCE::Channel>.
+
+=over 3
+
+=item new
+
  use MCE::Channel;
 
  my $chnl = MCE::Channel->new( impl => 'Simple' );
 
-The API is described in L<MCE::Channel>.
+=back
+
+=head1 QUEUE-LIKE BEHAVIOR
+
+=over 3
+
+=item enqueue
+
+=item dequeue
+
+=item dequeue_nb
+
+=item end
+
+=back
+
+=head1 TWO-WAY IPC - PRODUCER TO CONSUMER
+
+=over 3
+
+=item send
+
+=item recv
+
+=item recv_nb
+
+=back
+
+=head1 TWO-WAY IPC - CONSUMER TO PRODUCER
+
+=over 3
+
+=item send2
+
+=item recv2
+
+=item recv2_nb
+
+=back
 
 =head1 AUTHOR
 
