@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.843';
+our $VERSION = '1.865';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
@@ -22,11 +22,10 @@ use MCE;
 
 our @CARP_NOT = qw( MCE );
 
-my $_has_threads = $INC{'threads.pm'} ? 1 : 0;
-my $_tid = $_has_threads ? threads->tid() : 0;
+my $_tid = $INC{'threads.pm'} ? threads->tid() : 0;
 
 sub CLONE {
-   $_tid = threads->tid() if $_has_threads;
+   $_tid = threads->tid() if $INC{'threads.pm'};
 }
 
 ###############################################################################
@@ -350,7 +349,7 @@ MCE::Loop - MCE model for building parallel loops
 
 =head1 VERSION
 
-This document describes MCE::Loop version 1.843
+This document describes MCE::Loop version 1.865
 
 =head1 DESCRIPTION
 
@@ -449,9 +448,13 @@ inside the block. Hence, the block is called once per each item.
  mce_loop { do_work($_) } [ [ 0, 1 ], [ 0, 2 ], ... ];
  mce_loop { do_work($_) } \@deeply_list;
 
- ## File_path, glob_ref, or scalar_ref
- mce_loop_f { chomp; do_work($_) } "/path/to/file";
+ ## File path, glob ref, IO::All::{ File, Pipe, STDIO } obj, or scalar ref
+ ## Workers read directly and not involve the manager process
+ mce_loop_f { chomp; do_work($_) } "/path/to/file"; # efficient
+
+ ## Involves the manager process, therefore slower
  mce_loop_f { chomp; do_work($_) } $file_handle;
+ mce_loop_f { chomp; do_work($_) } $io;
  mce_loop_f { chomp; do_work($_) } \$scalar;
 
  ## Sequence of numbers (begin, end [, step, format])
@@ -516,7 +519,7 @@ The following list options which may be overridden when loading the module.
  ;
 
 From MCE 1.8 onwards, Sereal 3.015+ is loaded automatically if available.
-Specify C<Sereal => 0> to use Storable instead.
+Specify C<< Sereal => 0 >> to use Storable instead.
 
  use MCE::Loop Sereal => 0;
 
@@ -630,10 +633,13 @@ Input data may be defined using a list, an array ref, or a hash ref.
 The fastest of these is the /path/to/file. Workers communicate the next offset
 position among themselves with zero interaction by the manager process.
 
+C<IO::All> { File, Pipe, STDIO } is supported since MCE 1.845.
+
  # $_ contains the line when chunk_size => 1
 
  mce_loop_f { $_ } "/path/to/file";  # faster
  mce_loop_f { $_ } $file_handle;
+ mce_loop_f { $_ } $io;              # IO::All
  mce_loop_f { $_ } \$scalar;
 
  # chunking, any chunk_size => 1 or greater
