@@ -11,13 +11,21 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.843';
+our $VERSION = '1.865';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
 use Carp ();
 
+## global Mutex used by MCE, MCE::Child, and MCE::Hobo inside threads
+## on UNIX platforms
+
+if ( $INC{'threads.pm'} && $^O !~ /mswin|mingw|msys|cygwin/i ) {
+    $MCE::_GMUTEX = MCE::Mutex->new( impl => 'Channel' );
+    MCE::Mutex::Channel::_save_for_global_cleanup($MCE::_GMUTEX);
+}
+
 sub new {
     my ($class, %argv) = @_;
     my $impl = defined($argv{'impl'})
@@ -25,7 +33,7 @@ sub new {
 
     $impl = ucfirst( lc $impl );
 
-    eval "require MCE::Mutex::$impl; 1" ||
+    eval "require MCE::Mutex::$impl; 1;" ||
         Carp::croak("Could not load Mutex implementation '$impl': $@");
 
     my $pkg = 'MCE::Mutex::'.$impl;
@@ -45,7 +53,7 @@ sub timedwait {
 
     local $@; local $SIG{'ALRM'} = sub { alarm 0; die "timed out\n" };
 
-    eval { alarm $timeout || 1; $obj->lock_exclusive };
+    eval { alarm $timeout || 1; $obj->lock_exclusive; };
 
     alarm 0;
 
@@ -68,7 +76,7 @@ MCE::Mutex - Locking for Many-Core Engine
 
 =head1 VERSION
 
-This document describes MCE::Mutex version 1.843
+This document describes MCE::Mutex version 1.865
 
 =head1 SYNOPSIS
 
@@ -174,7 +182,8 @@ The method C<lock_exclusive> is an alias for C<lock>, available since 1.822.
 =head2 $mutex->lock_shared ( void )
 
 Like C<lock_exclusive>, but attempts to grab a shared lock instead.
-For non-Fcntl implementations, C<lock_shared> is an alias for C<lock>.
+The C<lock_shared> method is an alias to C<lock> otherwise for non-Fcntl
+implementations.
 
 Current API available since 1.822.
 
