@@ -9,43 +9,30 @@ package MCE::Signal;
 use strict;
 use warnings;
 
-no warnings qw( threads recursion uninitialized );
+no warnings qw( threads recursion uninitialized once );
 
-our $VERSION = '1.703';
+our $VERSION = '1.865';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 
 our ($display_die_with_localtime, $display_warn_with_localtime);
 our ($main_proc_id, $prog_name, $tmp_dir);
-my  ($_has_threads);
+
+tie $tmp_dir, 'MCE::Signal::_tmpdir';
 
 use Carp ();
 
 BEGIN {
    $main_proc_id =  $$;
-
    $prog_name    =  $0;
    $prog_name    =~ s{^.*[\\/]}{}g;
    $prog_name    =  'perl' if ($prog_name eq '-e' || $prog_name eq '-');
 
-   if ($^O eq 'MSWin32' && !defined $threads::VERSION) {
-      local $@; local $SIG{__DIE__} = \&_NOOP;
-      eval 'use threads; use threads::shared';
-   }
-   elsif (defined $threads::VERSION) {
-      unless (defined $threads::shared::VERSION) {
-         local $@; local $SIG{__DIE__} = \&_NOOP;
-         eval 'use threads::shared';
-      }
-   }
-
-   $_has_threads = $INC{'threads/shared.pm'} ? 1 : 0;
+   return;
 }
 
-use File::Path ();
-use Time::HiRes qw( sleep time );
-use Fcntl qw( :flock );
 use base qw( Exporter );
+use Time::HiRes ();
 
 our @EXPORT_OK = qw( $tmp_dir sys_cmd stop_and_exit );
 our %EXPORT_TAGS = (
@@ -53,7 +40,10 @@ our %EXPORT_TAGS = (
    tmp_dir => [ qw( $tmp_dir ) ]
 );
 
-sub _NOOP {}
+END {
+   MCE::Signal->stop_and_exit($?)
+      if ($$ == $main_proc_id && !$MCE::Signal::KILLED && !$MCE::Signal::STOPPED);
+}
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -67,31 +57,28 @@ sub _flag  { 1 }
 
 my $_is_MSWin32   = ($^O eq 'MSWin32') ? 1 : 0;
 my $_keep_tmp_dir = 0;
-my $_no_sigmsg    = 0;
+my $_use_dev_shm  = 0;
 my $_no_kill9     = 0;
 my $_imported;
 
 sub import {
+   my $_class = shift;
+   return if $_imported++;
 
-   my $_class = shift; return if ($_imported++);
-
-   my @_export_args = ();
-   my $_no_setpgrp  = 0;
-   my $_setpgrp     = 0;
-   my $_use_dev_shm = 0;
+   my ($_no_setpgrp, $_no_sigmsg, $_setpgrp, @_export_args) = (0, 0, 0);
 
    while (my $_arg = shift) {
-
+      $_setpgrp      = _flag() and next if ($_arg eq '-setpgrp');
       $_keep_tmp_dir = _flag() and next if ($_arg eq '-keep_tmp_dir');
-      $_no_sigmsg    = _flag() and next if ($_arg eq '-no_sigmsg');
+      $_use_dev_shm  = _flag() and next if ($_arg eq '-use_dev_shm');
       $_no_kill9     = _flag() and next if ($_arg eq '-no_kill9');
 
+      # deprecated options for backwards compatibility
       $_no_setpgrp   = _flag() and next if ($_arg eq '-no_setpgrp');
-      $_setpgrp      = _flag() and next if ($_arg eq '-setpgrp');
-
-      $_use_dev_shm  = _flag() and next if ($_arg eq '-use_dev_shm');
+      $_no_sigmsg    = _flag() and next if ($_arg eq '-no_sigmsg');
 
       _usage($_arg) if ($_arg =~ /^-/);
+
       push @_export_args, $_arg;
    }
 
@@ -99,32 +86,10 @@ sub import {
    Exporter::import($_class, @_export_args);
 
    ## Sets the current process group for the current process.
-   setpgrp(0,0) if ($_setpgrp == 1 && $^O ne 'MSWin32');
+   setpgrp(0,0) if ($_setpgrp == 1 && !$_is_MSWin32);
 
-   my ($_tmp_dir_base, $_count); $_count = 0;
-
-   if (exists $ENV{TEMP} && -d $ENV{TEMP} && -w $ENV{TEMP}) {
-      if ($_is_MSWin32) {
-         $_tmp_dir_base = $ENV{TEMP} . '/mce';
-         mkdir $_tmp_dir_base unless (-d $_tmp_dir_base);
-      } else {
-         $_tmp_dir_base = $ENV{TEMP};
-      }
-   }
-   else {
-      $_tmp_dir_base = ($_use_dev_shm && -d '/dev/shm' && -w '/dev/shm')
-         ? '/dev/shm' : '/tmp';
-   }
-
-   _croak("Error: MCE::Signal: ($_tmp_dir_base) is not writeable")
-      if (! exists $ENV{'MOBASTARTUPDIR'} && ! -w $_tmp_dir_base);
-
-   ## Remove tainted'ness from $tmp_dir.
-   ($tmp_dir) = "$_tmp_dir_base/$prog_name.$$.$_count" =~ /(.*)/;
-
-   while ( !(mkdir $tmp_dir, 0770) ) {
-      ($tmp_dir) = ("$_tmp_dir_base/$prog_name.$$.".(++$_count)) =~ /(.*)/;
-   }
+   ## Make tmp_dir if caller requested it.
+   _make_tmpdir() if ($_use_dev_shm || grep /tmp_dir/, @_export_args);
 
    return;
 }
@@ -135,43 +100,177 @@ sub import {
 ##
 ###############################################################################
 
-my ($_mce_sess_dir_ref, $_mce_spawned_ref);
+## Set traps to catch signals.
+if ( !$_is_MSWin32 ) {
+   $SIG{HUP}  = \&stop_and_exit;  # UNIX SIG  1
+   $SIG{INT}  = \&stop_and_exit;  # UNIX SIG  2
+   $SIG{PIPE} = \&stop_and_exit;  # UNIX SIG 13
+   $SIG{QUIT} = \&stop_and_exit;  # UNIX SIG  3
+   $SIG{TERM} = \&stop_and_exit;  # UNIX SIG 15
 
-sub _set_session_vars {
-   ($_mce_sess_dir_ref, $_mce_spawned_ref) = @_;
-   return;
+   ## MCE handles the reaping of its children.
+   $SIG{CHLD} = 'DEFAULT';
 }
 
-## Set traps to catch signals.
-$SIG{XCPU} = \&stop_and_exit if (exists $SIG{XCPU});   ## UNIX SIG 24
-$SIG{XFSZ} = \&stop_and_exit if (exists $SIG{XFSZ});   ## UNIX SIG 25
+my $_safe_clean = 0;
 
-$SIG{HUP}  = \&stop_and_exit;                          ## UNIX SIG  1
-$SIG{INT}  = \&stop_and_exit;                          ## UNIX SIG  2
-$SIG{PIPE} = \&stop_and_exit;                          ## UNIX SIG 13
-$SIG{QUIT} = \&stop_and_exit;                          ## UNIX SIG  3
-$SIG{TERM} = \&stop_and_exit;                          ## UNIX SIG 15
+sub _make_tmpdir {
+   my ($_count, $_tmp_base_dir) = (0);
 
-## For a more reliable MCE, $SIG{CHLD} is set to 'DEFAULT'. MCE handles
-## the reaping of its children, especially when running multiple MCEs
-## simultaneously.
-##
-$SIG{CHLD} = 'DEFAULT' unless ($_is_MSWin32);
+   return $tmp_dir if (defined $tmp_dir && -d $tmp_dir && -w _);
+
+   if ($ENV{TEMP} && -d $ENV{TEMP} && -w _) {
+      if ($^O =~ /mswin|mingw|msys|cygwin/i) {
+         $_tmp_base_dir = $ENV{TEMP} . '/Perl-MCE';
+         mkdir $_tmp_base_dir unless -d $_tmp_base_dir;
+      } else {
+         $_tmp_base_dir = $ENV{TEMP};
+      }
+   }
+   else {
+      $_tmp_base_dir = ($_use_dev_shm && -d '/dev/shm' && -w _)
+         ? '/dev/shm' : '/tmp';
+   }
+
+   _croak("Error: MCE::Signal: ($_tmp_base_dir) is not writeable")
+      if (! exists $ENV{'MOBASTARTUPDIR'} && ! -w $_tmp_base_dir);
+
+   ## Remove tainted'ness from $tmp_dir.
+   ($tmp_dir) = "$_tmp_base_dir/$prog_name.$$.$_count" =~ /(.*)/;
+
+   while ( !(mkdir $tmp_dir, 0770) ) {
+      ($tmp_dir) = ("$_tmp_base_dir/$prog_name.$$.".(++$_count)) =~ /(.*)/;
+   }
+
+   $_safe_clean = 1;
+
+   return $tmp_dir;
+}
+
+sub _remove_tmpdir {
+   return if (!defined $tmp_dir || $tmp_dir eq '' || ! -d $tmp_dir);
+
+   if ($_keep_tmp_dir == 1) {
+      print {*STDERR} "$prog_name: saved tmp_dir = $tmp_dir\n";
+   }
+   elsif ($_safe_clean) {
+      if ($ENV{'TEMP'} && $^O =~ /mswin|mingw|msys|cygwin/i) {
+         ## remove tainted'ness
+         my ($_dir) = $ENV{'TEMP'} =~ /(.*)/;
+         chdir $_dir if -d $_dir;
+      }
+      rmdir $tmp_dir;
+      if (-d $tmp_dir) {
+         local $@; local $SIG{__DIE__};
+         eval 'require File::Path; File::Path::rmtree($tmp_dir)';
+      }
+   }
+
+   $tmp_dir = undef;
+}
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## Call stop_and_exit when exiting the script.
+## Stops execution, removes temp directory and exits cleanly.
+##
+## Provides safe reentrant logic for parent and child processes.
+## The $main_proc_id variable is defined above.
 ##
 ###############################################################################
 
-END {
-   my $_exit_status = $?;
+BEGIN {
+   $MCE::Signal::IPC = 0;   # 1 = defer signal_handling until completed IPC
+   $MCE::Signal::SIG = '';  # signal received during IPC in MCE::Shared 1.863
+}
 
-   MCE::Signal->_shutdown_mce($_exit_status)
-      if (defined $_mce_spawned_ref);
+sub defer {
+   $MCE::Signal::SIG = $_[0] if $_[0];
+   return;
+}
 
-   MCE::Signal->stop_and_exit($_exit_status)
-      if ($$ == $main_proc_id);
+my %_sig_name_lkup = map { $_ => 1 } qw(
+   __DIE__ HUP INT PIPE QUIT TERM __WARN__
+);
+
+my $_count = 0;
+
+my $_handler_count = $INC{'threads/shared.pm'}
+   ? threads::shared::share($_count)
+   : \$_count;
+
+sub stop_and_exit {
+   shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
+   return MCE::Signal::defer($_[0]) if $MCE::Signal::IPC;
+
+   my ($_exit_status, $_is_sig, $_sig_name) = ($?, 0, $_[0] || 0);
+   $SIG{__DIE__} = $SIG{__WARN__} = sub {};
+
+   if (exists $_sig_name_lkup{$_sig_name}) {
+      $_exit_status = $MCE::Signal::KILLED = $_is_sig = 1;
+      $_exit_status = 255, $_sig_name = 'TERM' if ($_sig_name eq '__DIE__');
+      $_exit_status = 0 if ($_sig_name eq 'PIPE');
+      $SIG{INT} = $SIG{$_sig_name} = sub {};
+   }
+   else {
+      $_exit_status = $_sig_name if ($_sig_name =~ /^\d+$/);
+      $MCE::Signal::STOPPED = 1;
+   }
+
+   ## Main process.
+   if ($$ == $main_proc_id) {
+
+      if (++${ $_handler_count } == 1) {
+         ## Kill process group if signaled.
+         if ($_is_sig == 1) {
+            ($_sig_name eq 'PIPE')
+               ? CORE::kill('PIPE', $_is_MSWin32 ? -$$ : -getpgrp)
+               : CORE::kill('INT' , $_is_MSWin32 ? -$$ : -getpgrp);
+
+            if ($_sig_name eq 'PIPE') {
+               for my $_i (1..2) { Time::HiRes::sleep(0.015); }
+            } else {
+               for my $_i (1..3) { Time::HiRes::sleep(0.060); }
+            }
+         }
+
+         ## Remove temp directory.
+         _remove_tmpdir() if defined($tmp_dir);
+
+         ## Signal process group to die.
+         if ($_is_sig == 1) {
+            if ($_sig_name eq 'INT' && -t STDIN) { ## no critic
+               print {*STDERR} "\n";
+            }
+            if ($INC{'threads.pm'} && ($] lt '5.012000' || threads->tid())) {
+               ($_no_kill9 == 1 || $_sig_name eq 'PIPE')
+                  ? CORE::kill('INT', $_is_MSWin32 ? -$$ : -getpgrp)
+                  : CORE::kill('KILL', -$$);
+            }
+            else {
+               CORE::kill('INT', $_is_MSWin32 ? -$$ : -getpgrp);
+            }
+         }
+      }
+   }
+
+   ## Child processes.
+   elsif ($_is_sig) {
+
+      ## Windows support, from nested workers.
+      if ($_is_MSWin32) {
+         _remove_tmpdir() if defined($tmp_dir);
+         CORE::kill('KILL', $main_proc_id, -$$);
+      }
+
+      ## Real child processes.
+      else {
+         CORE::kill($_sig_name, $main_proc_id, -$$);
+         CORE::kill('KILL', -$$, $$);
+      }
+   }
+
+   ## Exit with status.
+   CORE::exit($_exit_status);
 }
 
 ###############################################################################
@@ -189,7 +288,6 @@ END {
 ###############################################################################
 
 sub sys_cmd {
-
    shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
 
    _croak('MCE::Signal::sys_cmd: no arguments were specified') if (@_ == 0);
@@ -200,211 +298,15 @@ sub sys_cmd {
 
    ## Kill the process group if command caught SIGINT or SIGQUIT.
 
-   CORE::kill('INT',  $main_proc_id, ($_is_MSWin32 ? -$$ : -getpgrp))
+   CORE::kill('INT',  $main_proc_id, $_is_MSWin32 ? -$$ : -getpgrp)
       if $_sig_no == 2;
 
-   CORE::kill('QUIT', $main_proc_id, ($_is_MSWin32 ? -$$ : -getpgrp))
+   CORE::kill('QUIT', $main_proc_id, $_is_MSWin32 ? -$$ : -getpgrp)
       if $_sig_no == 3;
 
    return $_exit_status;
 }
 
-###############################################################################
-## ----------------------------------------------------------------------------
-## Stops execution, removes temp directory and exits cleanly.
-##
-## Provides safe reentrant logic for both parent and child processes.
-## The $main_proc_id variable is defined above.
-##
-###############################################################################
-
-my %_sig_name_lkup = map { $_ => 1 } qw(
-   __DIE__ __WARN__ HUP INT PIPE QUIT TERM CHLD XCPU XFSZ
-);
-
-my $_handler_cnt : shared = 0;
-
-sub stop_and_exit {
-
-   shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
-
-   my $_is_main_proc = ($$ == $main_proc_id);
-   my $_sig_name     = $_[0] || 0;
-   my $_exit_status  = $?;
-   my $_is_sig       = 0;
-
-   if (exists $_sig_name_lkup{$_sig_name}) {
-      $_exit_status     = $MCE::Signal::KILLED = $_is_sig = 1;
-      $SIG{$_sig_name}  = \&_NOOP;
-      $_mce_spawned_ref = undef;
-   }
-   else {
-      $_exit_status = $_sig_name if ($_sig_name =~ /^\d+$/);
-   }
-
-   $SIG{INT} = $SIG{__DIE__} = $SIG{__WARN__} = \&_NOOP;
-
-   ## -------------------------------------------------------------------------
-
-   ## For the main process.
-   if ($_is_main_proc) {
-      lock $_handler_cnt if $_has_threads;
-
-      if (++$_handler_cnt == 1) {
-         open my $_FH, '>>', "$tmp_dir/stopped";
-         close   $_FH;
-
-         local $\ = undef;
-
-         ## Display message and kill process group if signaled.
-         if ($_is_sig == 1) {
-            my $_err_msg = undef;
-
-            if ($_sig_name eq 'XCPU') {
-               $_err_msg  = 'exceeded CPU time limit, exiting';
-               $_sig_name = 'INT';
-            }
-            elsif ($_sig_name eq 'XFSZ') {
-               $_err_msg  = 'exceeded file size limit, exiting';
-               $_sig_name = 'INT';
-            }
-            elsif ($_sig_name eq 'INT' && -f "$tmp_dir/died") {
-               $_err_msg  = 'caught signal (__DIE__), exiting';
-            }
-            elsif ($_sig_name eq '__DIE__') {
-               $_err_msg  = 'caught signal (__DIE__), exiting';
-               $_sig_name = 'INT';
-            }
-            elsif ($_sig_name ne 'PIPE') {
-               $_err_msg  = "caught signal ($_sig_name), exiting";
-            }
-
-            if ($_err_msg && $_no_sigmsg == 0) {
-               print {*STDERR} "\n## $prog_name: $_err_msg\n";
-            }
-
-            open my $_FH, '>', "$tmp_dir/killed";
-            close   $_FH;
-
-            CORE::kill($_sig_name, $_is_MSWin32 ? -$$ : -getpgrp);
-
-            ## Pause a bit.
-            if ($_sig_name eq 'PIPE') {
-               sleep 0.015 for (1..2);
-            } else {
-               sleep 0.065 for (1..3);
-            }
-         }
-
-         ## Remove temp directory.
-         if (defined $tmp_dir && $tmp_dir ne '' && -d $tmp_dir) {
-            if (defined $_mce_sess_dir_ref) {
-               for my $_sess_dir (keys %{ $_mce_sess_dir_ref }) {
-                  File::Path::rmtree($_sess_dir);
-                  delete $_mce_sess_dir_ref->{$_sess_dir};
-               }
-            }
-            if ($_keep_tmp_dir == 1) {
-               print {*STDERR} "$prog_name: saved tmp_dir = $tmp_dir\n";
-            } elsif ($tmp_dir ne '/tmp' && $tmp_dir ne '/var/tmp') {
-               File::Path::rmtree($tmp_dir);
-            }
-            $tmp_dir = undef;
-         }
-
-         ## Signal process group to die.
-         if ($_is_sig == 1) {
-            if ($_sig_name eq 'PIPE' && $INC{'MCE/Hobo.pm'}) {
-               CORE::kill('QUIT', $_is_MSWin32 ? -$$ : -getpgrp);
-            }
-            if ($_sig_name ne 'PIPE' && $_no_sigmsg == 0) {
-               print {*STDERR} "\n";
-            }
-            ($_no_kill9 == 1 || $_sig_name eq 'PIPE')
-               ? CORE::kill('INT', $_is_MSWin32 ? -$$ : -getpgrp)
-               : CORE::kill('KILL', -$$, $main_proc_id);
-         }
-      }
-   }
-
-   ## Otherwise, for child processes.
-   elsif ($_is_sig == 1 && -d $tmp_dir) {
-      if (++$_handler_cnt == 1) {
-
-         ## Obtain lock.
-         open my $CHILD_LOCK, '+>>', "$tmp_dir/child.lock";
-         flock $CHILD_LOCK, LOCK_EX;
-
-         ## Notify the main process that I've died.
-         if ($_sig_name eq '__DIE__' && ! -f "$tmp_dir/died") {
-            local $@; eval {
-               open my $_FH, '>>', "$tmp_dir/died";
-               close   $_FH;
-            };
-         }
-
-         ## Signal process group to terminate.
-         if (! -f "$tmp_dir/killed" && ! -f "$tmp_dir/stopped") {
-            local $@; eval {
-               open my $_FH, '>>', "$tmp_dir/killed";
-               close   $_FH;
-            };
-            ($_sig_name eq 'PIPE')
-               ? CORE::kill('PIPE', $main_proc_id, -$$)
-               : CORE::kill('INT', $main_proc_id, -$$);
-         }
-
-         ## Release lock.
-         flock $CHILD_LOCK, LOCK_UN;
-         close $CHILD_LOCK;
-      }
-   }
-
-   ## -------------------------------------------------------------------------
-
-   ## Exit with status.
-   if ($_is_sig == 1 && $_no_kill9 == 0) {
-      sleep 0.065 for (1..5);
-   }
-
-   CORE::exit($_exit_status);
-}
-
-###############################################################################
-## ----------------------------------------------------------------------------
-## Shutdown MCEs that were previously initiated by this process ID and are
-## still running.
-##
-###############################################################################
-
-sub _shutdown_mce {
-
-   shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
-   my $_exit_status = $_[0] || $?;
-
-   if (defined $_mce_spawned_ref && !$MCE::Signal::KILLED) {
-      my $_tid = ($INC{'threads.pm'}) ? threads->tid() : '';
-         $_tid = '' unless defined $_tid;
-
-      for my $_mce_sid (keys %{ $_mce_spawned_ref }) {
-         if ($_mce_spawned_ref->{$_mce_sid}->wid()) {
-            if ($_mce_spawned_ref->{$_mce_sid}->pid() == $$) {
-               $_mce_spawned_ref->{$_mce_sid}->exit($_exit_status);
-            }
-         }
-         else {
-            if ($_mce_sid =~ /\A$$\.$_tid\./) {
-               $_mce_spawned_ref->{$_mce_sid}->shutdown();
-            }
-         }
-
-         delete $_mce_spawned_ref->{$_mce_sid};
-      }
-   }
-
-   return;
-}
-
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Signal handlers for __DIE__ & __WARN__ utilized by MCE.
@@ -412,7 +314,6 @@ sub _shutdown_mce {
 ###############################################################################
 
 sub _die_handler {
-
    shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
 
    if (!defined $^S || $^S) {
@@ -448,11 +349,10 @@ sub _die_handler {
       }
    }
 
-   MCE::Signal->stop_and_exit('__DIE__');
+   MCE::Signal::stop_and_exit('__DIE__');
 }
 
 sub _warn_handler {
-
    shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
 
    ## Ignore thread warnings during exiting.
@@ -484,6 +384,46 @@ sub _warn_handler {
 
 1;
 
+###############################################################################
+## ----------------------------------------------------------------------------
+## TIE scalar package for making $MCE::Signal::tmp_dir on demand.
+##
+###############################################################################
+
+package MCE::Signal::_tmpdir;
+
+sub TIESCALAR {
+   my $_class = shift;
+   bless \do{ my $o = defined $_[0] ? shift : undef }, $_class;
+}
+
+sub STORE {
+   ${ $_[0] } = $_[1];
+
+   $_safe_clean = 0 if ( length $_[1] < 9 );
+   $_safe_clean = 0 if ( $ENV{'TEMP'} && $ENV{'TEMP'} eq $_[1] );
+   $_safe_clean = 0 if ( $_[1] =~ m{[\\/](?:etc|bin|lib|sbin)} );
+   $_safe_clean = 0 if ( $_[1] =~ m{[\\/](?:temp|tmp)[\\/]?$}i );
+
+   $_[1];
+}
+
+sub FETCH {
+   if (!defined ${ $_[0] }) {
+      my $_caller = caller();
+      if ($_caller ne 'MCE' && $_caller ne 'MCE::Signal') {
+         if ($INC{'MCE.pm'} && MCE->wid() > 0) {
+            ${ $_[0] } = MCE->tmp_dir();
+         } else {
+            ${ $_[0] } = MCE::Signal::_make_tmpdir();
+         }
+      }
+   }
+   ${ $_[0] };
+}
+
+1;
+
 __END__
 
 ###############################################################################
@@ -498,7 +438,7 @@ MCE::Signal - Temporary directory creation/cleanup and signal handling
 
 =head1 VERSION
 
-This document describes MCE::Signal version 1.703
+This document describes MCE::Signal version 1.865
 
 =head1 SYNOPSIS
 
@@ -523,7 +463,7 @@ This document describes MCE::Signal version 1.703
 
 =head1 DESCRIPTION
 
-This package configures $SIG{HUP,INT,PIPE,QUIT,TERM,XCPU,XFSZ} to point to
+This package configures $SIG{ HUP, INT, PIPE, QUIT, and TERM } to point to
 stop_and_exit and creates a temporary directory. The main process and workers
 receiving said signals call stop_and_exit, which signals all workers to
 terminate, removes the temporary directory unless -keep_tmp_dir is specified,
@@ -531,9 +471,7 @@ and terminates itself.
 
 The location of the temp directory resides under $ENV{TEMP} if defined,
 otherwise /dev/shm if writeable and -use_dev_shm is specified, or /tmp.
-
-The temp dir resides under $ENV{TEMP}/mce/ for native Perl on Microsoft
-Windows.
+On Windows, the temp directory is made under $ENV{TEMP}/Perl-MCE/.
 
 As of MCE 1.405, MCE::Signal no longer calls setpgrp by default. Pass the
 -setpgrp option to MCE::Signal to call setpgrp.
@@ -550,12 +488,9 @@ The following are available options and their meanings.
                      A message is displayed with the location afterwards
 
  -use_dev_shm      - Create the temporary directory under /dev/shm
-
- -no_sigmsg        - Do not display a message when receiving a signal
  -no_kill9         - Do not kill -9 after receiving a signal to terminate
 
  -setpgrp          - Calls setpgrp to set the process group for the process
-
                      This option ensures all workers terminate when reading
                      STDIN for MCE releases 1.511 and below.
 
@@ -584,7 +519,7 @@ Pass 'INT' to terminate a spawned or running MCE session.
 
 =head2 sys_cmd ( $command )
 
-The system function in Perl ignores SIGNINT and SIGQUIT. These 2 signals are
+The system function in Perl ignores SIGINT and SIGQUIT. These 2 signals are
 sent to the command being executed via system() but not back to the underlying
 Perl script. For this reason, sys_cmd was added to MCE::Signal.
 
@@ -596,6 +531,73 @@ Perl script. For this reason, sys_cmd was added to MCE::Signal.
 
  my $exit_status = sys_cmd($command);
 
+=head1 DEFER SIGNAL
+
+=head2 defer ( $signal )
+
+Returns immediately inside a signal handler if signaled during IPC.
+The signal is deferred momentarily and re-signaled automatically upon
+completing IPC. Currently, all IPC related methods in C<MCE::Shared> and
+one method C<send2> in C<MCE::Channel> set the flag C<$MCE::Signal::IPC>
+before initiating IPC.
+
+Current API available since 1.863.
+
+ sub sig_handler {
+    return MCE::Signal::defer($_[0]) if $MCE::Signal::IPC;
+    ...
+ }
+
+In a nutshell, C<defer> helps safeguard IPC from stalling between workers
+and the shared manager-process. The following is a demonstration for Unix
+platforms. Deferring the signal inside the C<WINCH> handler prevents the
+app from eventually failing while resizing the window.
+
+ use strict;
+ use warnings;
+
+ use MCE::Hobo;
+ use MCE::Shared;
+ use Time::HiRes 'sleep';
+
+ my $count = MCE::Shared->scalar(0);
+ my $winch = MCE::Shared->scalar(0);
+ my $done  = MCE::Shared->scalar(0);
+
+ $SIG{WINCH} = sub {
+    # defer signal if signaled during IPC
+    return MCE::Signal::defer($_[0]) if $MCE::Signal::IPC;
+
+    # mask signal handler
+    local $SIG{$_[0]} = 'IGNORE';
+
+    printf "inside winch handler %d\n", $winch->incr;
+ };
+
+ $SIG{INT} = sub {
+    # defer signal if signaled during IPC
+    return MCE::Signal::defer($_[0]) if $MCE::Signal::IPC;
+
+    # set flag for workers to leave loop
+    $done->set(1);
+ };
+
+ sub task {
+    while ( ! $done->get ) {
+       $count->incr;
+       sleep 0.03;
+    };
+ }
+
+ print "Resize the terminal window continuously.\n";
+ print "Press Ctrl-C to stop.\n";
+
+ MCE::Hobo->create('task') for 1..8;
+ sleep 0.015 until $done->get;
+ MCE::Hobo->wait_all;
+
+ printf "\ncount incremented %d times\n\n", $count->get;
+
 =head1 INDEX
 
 L<MCE|MCE>, L<MCE::Core>
